<form action="" method="get"{% if class %} class="{{ class }}"{% endif %}{% if rel %} rel="{{ rel }}"{% endif %}>
    <input type="hidden" name="{{ with }}" value="{{ query }}" />
    <input type="submit" value="{{ label }}" title="{{ title }}" />
</form>